"""
Backends for converting data arrays using popular libraries.

Each backend is implemented in its own module and should be imported
from there directly to avoid unnecessary dependencies.

Example:
    from pixel_canon.backends.numpy_backend import convert_numpy
"""
# This file makes 'backends' a package.
